import { useCallback, useEffect, useState } from "react";
import Select from "../../components/Select";
import Slider from "../../components/Slider";
import useLocalStorage from "../../components/useLocalStorage";
import { Tool, toolsDatabase } from "../../toolsDatabase";
import { v4 as uuid4 } from "uuid";

const frezowaniaOpcje = {
	name: "rodzajFrezowania",
	label: "Rodzaj Frezowania",
	options: ["Czołowe", "Walcowo Czołowe", "Rowków"],
	images: ["/czolowe_sandvik.webp", "/walcowo-czolowe_sandvik.webp", "/rowkowe_sandvik.webp"],
};
const dropdowns = [
	{
		name: "rodzajObrobki",
		label: "Rodzaj Obróbki",
		options: ["Zgrubna", "Wykończeniowa", "Wstępna i wykańczająca"],
	},
	{
		name: "material",
		label: "Materiał",
		options: ["Stal niskostopowa", "Stal nierdzewna", "Żeliwo"],
	},
	{
		name: "tolerancja",
		label: "Klasa dokładności wykonania",
		options: ["IT5", "IT6", "IT7", "IT8", "IT9", "IT10", "IT11"],
	},
	{
		name: "chropowatosc",
		label: "Chropowatość",
		options: ["Ra 0,1", "Ra 0,2", "Ra 0,4", "Ra 0,8", "Ra 1,6", "Ra 3,2", "Ra 6,3", "Ra 12,5", "Ra 25", "Ra 50"],
	},
];
interface IInput {
	rodzajFrezowania: number;
	rodzajObrobki: number;
	material: number;
	glebokoscSkrawania: number;
	szerokoscSkrawania: number;
	tolerancja: number;
	chropowatosc: number;
}
// interface IResults {
// 	predkoscObrotowaWrzeciona: number;
// 	predkoscSkrawania: number;
// 	predkoscPosuwu: number;
// 	posowNaOstrze: number;
// 	posowNaObrot: number;
// 	wydajnoscObjetosciowa: number;
// }

const HomePage = () => {
	// ----------------------------- STATE -----------------------------
	const [input, setInput] = useState<IInput>({
		// lists
		rodzajFrezowania: 0,
		rodzajObrobki: 0,
		material: 0,
		// variables
		glebokoscSkrawania: Math.floor(Math.random() * 50 + 1),
		szerokoscSkrawania: Math.floor(Math.random() * 50 + 1),
		tolerancja: 0,
		chropowatosc: 0,
	});
	const [result, setResult] = useLocalStorage<Tool[] | null>("results", []); // Znalazione narzędzie
	// const [result, setResult] = useState<Tool | null>(null); // Znalazione narzędzie
	const [calculating, setCalculating] = useState(false);
	const [notFound, setNotFound] = useState(false);
	// ----------------------------- FUNCTION --------------------------
	const handleChangeOfField = (key: keyof typeof input) => (e: React.ChangeEvent<HTMLSelectElement>) => {
		const value = Number(e.target.value);
		setInput((prev) => ({ ...prev, [key]: value }));
	};
	const matchesInput = (tool: Tool, input: IInput): boolean => {
		return (
			tool.rodzajFrezowania === input.rodzajFrezowania &&
			tool.rodzajObrobki === input.rodzajObrobki &&
			tool.material === input.material &&
			input.chropowatosc === tool.chropowatosc &&
			input.szerokoscSkrawania === tool.szerokoscSkrawania &&
			input.glebokoscSkrawania === tool.glebokoscSkrawania &&
			input.tolerancja === tool.tolerancja
		);
	};
	const checkInputInDatabse = useCallback(
		(input: IInput): void => {
			console.debug("checking...", input);

			const foundTool: Tool[] | undefined = toolsDatabase.tools.filter((tool) => matchesInput(tool, input));

			if (foundTool && typeof foundTool !== "undefined" && foundTool.length > 0) {
				console.debug("found", foundTool);
				setResult(foundTool);
			} else {
				console.debug("not found");
				setResult(null);
				setNotFound(true);
			}
		},
		[setResult],
	);

	// ----------------------------- EFFECT ----------------------------
	useEffect(() => {
		if (calculating) {
			checkInputInDatabse(input);
		}
		setCalculating(false);
	}, [calculating, checkInputInDatabse, input]);
	// debug
	useEffect(() => {
		setInput({
			// lists
			rodzajFrezowania: 1,
			rodzajObrobki: 0,
			material: 1,
			// variables
			glebokoscSkrawania: 5,
			szerokoscSkrawania: 20,
			tolerancja: 3,
			chropowatosc: 2,
		});
		console.dir(input);
		return () => {
			setResult(null);
		};
	}, []);
	// ----------------------------- RENDER ----------------------------
	return (
		<>
			<div className="p-6 bg-white rounded shadow-lg lg:w-[800px] space-y-4 mb-4 mt-4 lg:ml-[50%] lg:translate-x-[-50%] ">
				<h1 className="text-3xl mb-6 text-center text-gray-700">Wybór atrybutów</h1>

				<div className="mb-4">
					<label className="block text-gray-700 text-sm font-bold mb-2">{frezowaniaOpcje.label}</label>
					<div className="grid sm:grid-cols-3 grid-cols-2 gap-4 place-items-center">
						{frezowaniaOpcje.options.map((option, index) => (
							<div key={index}>
								<input
									type="radio"
									name="radio-1"
									className="radio"
									value={input.rodzajFrezowania}
									checked={input.rodzajFrezowania === index}
									onChange={() => {
										setInput((prev) => ({ ...prev, rodzajFrezowania: index }));
									}}
								/>
								<label className="text-sm ml-2 align-super">{option}</label>
								<img src={frezowaniaOpcje.images[index]} alt={option} className="max-w-[120px] mb-4 select-none" />
							</div>
						))}
					</div>
				</div>

				<div className="space-y-4">
					{dropdowns.map((dropdown) => (
						<div className="mb-4" key={dropdown.label}>
							<label className="block text-gray-700 text-sm font-bold mb-2">{dropdown.label}</label>
							<Select
								value={input[dropdown.label as keyof typeof input]}
								onChange={handleChangeOfField(dropdown.name as keyof typeof input)}
								options={dropdown.options}
								label={dropdown.label}
								disabledLabel={`Wybierz ${dropdown.label}`}
							/>
						</div>
					))}
				</div>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Głębokość Skrawania</label>
				<Slider
					range={{ min: 1, max: 50 }}
					value={input.glebokoscSkrawania}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, glebokoscSkrawania: Number(e!.target.value) }));
					}}
					suffix="mm"
				/>

				<label className="flex text-gray-700 text-sm font-bold mb-0">Szerokość Skrawania</label>
				<Slider
					range={{ min: 1, max: 50 }}
					value={input.szerokoscSkrawania}
					onChange={(e) => {
						setInput((prev) => ({ ...prev, szerokoscSkrawania: Number(e!.target.value) }));
					}}
					suffix="mm"
				/>
				<button
					className="btn btn-secondary w-full mt-4"
					type="submit"
					onClick={() => {
						setCalculating(true);
					}}
				>
					Dobierz narzędzia i parametry skrawania
				</button>
			</div>

			<div className="p-6 bg-white rounded shadow-lg lg:w-[800px] mb-4 lg:ml-[50%] lg:translate-x-[-50%]">
				{result ? (
					<>
						{result.map((tool) => {
							return (
								<div key={uuid4()}>
									<h2 className="text-2xl mb-1">{tool.name}</h2>
									<table className="table">
										<thead>
											<tr>
												<th className="px-4 py-2">Parametr</th>
												<th className="px-4 py-2">Wartość</th>
											</tr>
										</thead>
										<tbody>
											{Object.entries(tool).map(([key, value]) => {
												return (
													<tr>
														<td className="border px-4 py-2">{key}</td>
														<td className="border px-4 py-2">
															<input type="text" className="input w-full" value={value} />
														</td>
													</tr>
												);
											})}
										</tbody>
									</table>
									<button className="btn btn-neutral mt-4 mb-4">Zapisz zmiany jako nowe narzędzie</button>
									<div className="h-px bg-neutral-content mb-2"></div>
								</div>
							);
						})}
					</>
				) : notFound ? (
					<h2 className="text-xl">❗Nie znaleziono narzędzia dla podanych parametrów</h2>
				) : (
					<p className="text-neutral-content">Tutaj pojawią się wyniki</p>
				)}
			</div>
		</>
	);
};

export default HomePage;
